@@ -1,18 +1,6 @@
-# Copyright (c) 2003-2014 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """classes checker for Python code
 """
 from __future__ import generators
@@ -20,13 +8,13 @@ from __future__ import generators
 import sys
 from collections import defaultdict
 
+import six
+
 import astroid
 from astroid.bases import Generator, BUILTINS
 from astroid.exceptions import InconsistentMroError, DuplicateBasesError
 from astroid import objects
 from astroid.scoped_nodes import function_to_method
-from astroid import helpers
-
 from pylint.interfaces import IAstroidChecker
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import (
@@ -34,9 +22,10 @@ from pylint.checkers.utils import (
     overrides_a_method, check_messages, is_attr_private,
     is_attr_protected, node_frame_class, is_builtin_object,
     decorated_with_property, unimplemented_abstract_methods,
-    decorated_with)
+    decorated_with, class_is_abstract,
+    safe_infer, has_known_bases)
 from pylint.utils import deprecated_option, get_global_option
-import six
+
 
 if sys.version_info >= (3, 0):
     NEXT_METHOD = '__next__'
@@ -102,15 +91,6 @@ def _called_in_methods(func, klass, methods):
                     return True
     return False
 
-def class_is_abstract(node):
-    """return true if the given class node should be considered as an abstract
-    class
-    """
-    for method in node.methods():
-        if method.parent.frame() is node:
-            if method.is_abstract(pass_is_abstract=False):
-                return True
-    return False
 
 def _is_attribute_property(name, klass):
     """ Check if the given attribute *name* is a property
@@ -140,6 +120,36 @@ def _is_attribute_property(name, klass):
             return True
     return False
 
+def _has_bare_super_call(fundef_node):
+    for call in fundef_node.nodes_of_class(astroid.Call):
+        func = call.func
+        if (isinstance(func, astroid.Name) and
+                func.name == 'super' and
+                not call.args):
+            return True
+    return False
+
+def _safe_infer_call_result(node, caller, context=None):
+    """
+    Safely infer the return value of a function.
+
+    Returns None if inference failed or if there is some ambiguity (more than
+    one node has been inferred). Otherwise returns infered value.
+    """
+    try:
+        inferit = node.infer_call_result(caller, context=context)
+        value = next(inferit)
+    except astroid.InferenceError:
+        return  # inference failed
+    except StopIteration:
+        return  # no values infered
+    try:
+        next(inferit)
+        return  # there is ambiguity on the inferred node
+    except astroid.InferenceError:
+        return  # there is some kind of ambiguity
+    except StopIteration:
+        return value
 
 MSGS = {
     'F0202': ('Unable to check methods signature (%s / %s)',
@@ -254,7 +264,14 @@ MSGS = {
     'E0241': ('Duplicate bases for class %r',
               'duplicate-bases',
               'Used when a class has duplicate bases.'),
-
+    'R0202': ('Consider using a decorator instead of calling classmethod',
+              'no-classmethod-decorator',
+              'Used when a class method is defined without using the decorator '
+              'syntax.'),
+    'R0203': ('Consider using a decorator instead of calling staticmethod',
+              'no-staticmethod-decorator',
+              'Used when a static method is defined without using the decorator '
+              'syntax.'),
     }
 
 
@@ -276,11 +293,11 @@ class ClassChecker(BaseChecker):
     priority = -2
     # configuration options
     options = (('ignore-iface-methods',
-                # TODO(cpopa): remove this in Pylint 1.6.
                 deprecated_option(opt_type="csv",
                                   help_msg="This is deprecated, because "
-                                           "it is not used anymore.")
-               ),
+                                           "it is not used anymore.",
+                                  deprecation_msg="This option %r will be "
+                                                  "removed in Pylint 2.0")),
                ('defining-attr-methods',
                 {'default' : ('__init__', '__new__', 'setUp'),
                  'type' : 'csv',
@@ -325,7 +342,7 @@ a metaclass class method.'}
         self._accessed.append(defaultdict(list))
         self._check_bases_classes(node)
         # if not an exception or a metaclass
-        if node.type == 'class' and helpers.has_known_bases(node):
+        if node.type == 'class' and has_known_bases(node):
             try:
                 node.local_attr('__init__')
             except astroid.NotFoundError:
@@ -334,7 +351,6 @@ a metaclass class method.'}
         self._check_proper_bases(node)
         self._check_consistent_mro(node)
 
-    @check_messages('inconsistent-mro', 'duplicate-bases')
     def _check_consistent_mro(self, node):
         """Detect that a class has a consistent mro or duplicate bases."""
         try:
@@ -347,14 +363,13 @@ a metaclass class method.'}
             # Old style class, there's no mro so don't do anything.
             pass
 
-    @check_messages('inherit-non-class')
     def _check_proper_bases(self, node):
         """
         Detect that a class inherits something which is not
         a class or a type.
         """
         for base in node.bases:
-            ancestor = helpers.safe_infer(base)
+            ancestor = safe_infer(base)
             if ancestor in (astroid.YES, None):
                 continue
             if (isinstance(ancestor, astroid.Instance) and
@@ -366,8 +381,6 @@ a metaclass class method.'}
                 self.add_message('inherit-non-class',
                                  args=base.as_string(), node=node)
 
-    @check_messages('access-member-before-definition',
-                    'attribute-defined-outside-init')
     def leave_classdef(self, cnode):
         """close a class node:
         check that instance attributes are defined in __init__ and check
@@ -479,6 +492,8 @@ a metaclass class method.'}
         except astroid.NotFoundError:
             pass
 
+    visit_asyncfunctiondef = visit_functiondef
+
     def _check_slots(self, node):
         if '__slots__' not in node.locals:
             return
@@ -542,10 +557,11 @@ a metaclass class method.'}
                 return
             class_node = node.parent.frame()
             if (self._meth_could_be_func and node.type == 'method'
-                    and not node.name in PYMETHODS
+                    and node.name not in PYMETHODS
                     and not (node.is_abstract() or
                              overrides_a_method(class_node, node.name) or
-                             decorated_with_property(node))):
+                             decorated_with_property(node) or
+                             (six.PY3 and _has_bare_super_call(node)))):
                 self.add_message('no-self-use', node=node)
 
     def visit_attribute(self, node):
@@ -573,7 +589,7 @@ a metaclass class method.'}
         """ Check that the given assattr node
         is defined in the class slots.
         """
-        infered = helpers.safe_infer(node.expr)
+        infered = safe_infer(node.expr)
         if infered and isinstance(infered, astroid.Instance):
             klass = infered._proxied
             if '__slots__' not in klass.locals or not klass.newstyle:
@@ -604,17 +620,53 @@ a metaclass class method.'}
                     self.add_message('assigning-non-slot',
                                      args=(node.attrname, ), node=node)
 
-    @check_messages('protected-access')
+    @check_messages('protected-access', 'no-classmethod-decorator',
+                    'no-staticmethod-decorator')
     def visit_assign(self, assign_node):
+        self._check_classmethod_declaration(assign_node)
         node = assign_node.targets[0]
         if not isinstance(node, astroid.AssignAttr):
             return
 
         if self.is_first_attr(node):
             return
-
         self._check_protected_attribute_access(node)
 
+    def _check_classmethod_declaration(self, node):
+        """Checks for uses of classmethod() or staticmethod()
+
+        When a @classmethod or @staticmethod decorator should be used instead.
+        A message will be emitted only if the assignment is at a class scope
+        and only if the classmethod's argument belongs to the class where it
+        is defined.
+        `node` is an assign node.
+        """
+        if not isinstance(node.value, astroid.Call):
+            return
+
+        # check the function called is "classmethod" or "staticmethod"
+        func = node.value.func
+        if (not isinstance(func, astroid.Name) or
+                func.name not in ('classmethod', 'staticmethod')):
+            return
+
+        msg = ('no-classmethod-decorator' if func.name == 'classmethod' else
+               'no-staticmethod-decorator')
+        # assignment must be at a class scope
+        parent_class = node.scope()
+        if not isinstance(parent_class, astroid.ClassDef):
+            return
+
+        # Check if the arg passed to classmethod is a class member
+        classmeth_arg = node.value.args[0]
+        if not isinstance(classmeth_arg, astroid.Name):
+            return
+
+        method_name = classmeth_arg.name
+        if any(method_name == member.name
+               for member in parent_class.mymethods()):
+            self.add_message(msg, node=node.targets[0])
+
     def _check_protected_attribute_access(self, node):
         '''Given an attribute access node (set or get), check if attribute
         access is legitimate. Call _check_first_attr with node before calling
@@ -658,16 +710,13 @@ a metaclass class method.'}
                 #     b = property(lambda: self._b)
 
                 stmt = node.parent.statement()
-                try:
-                    if (isinstance(stmt, astroid.Assign) and
-                            (stmt in klass.body or klass.parent_of(stmt)) and
-                            isinstance(stmt.value, astroid.Call) and
-                            isinstance(stmt.value.func, astroid.Name) and
-                            stmt.value.func.name == 'property' and
-                            is_builtin_object(next(stmt.value.func.infer(), None))):
+                if (isinstance(stmt, astroid.Assign)
+                        and len(stmt.targets) == 1
+                        and isinstance(stmt.targets[0], astroid.AssignName)):
+                    name = stmt.targets[0].name
+                    if _is_attribute_property(name, klass):
                         return
-                except astroid.InferenceError:
-                    pass
+
                 self.add_message('protected-access', node=node, args=attrname)
 
     def visit_name(self, node):
@@ -867,7 +916,8 @@ a metaclass class method.'}
             except astroid.InferenceError:
                 continue
         for klass, method in six.iteritems(not_called_yet):
-            if klass.name == 'object' or method.parent.name == 'object':
+            cls = node_frame_class(method)
+            if klass.name == 'object' or (cls and cls.name == 'object'):
                 continue
             self.add_message('super-init-not-called', args=klass.name, node=node)
 
@@ -939,19 +989,27 @@ class SpecialMethodsChecker(BaseChecker):
                   'invalid number of parameters. If it has too few or '
                   'too many, it might not work at all.',
                   {'old_names': [('E0235', 'bad-context-manager')]}),
+        'E0303': ('__len__ does not return non-negative integer',
+                  'invalid-length-returned',
+                  'Used when an __len__ method returns something which is not a '
+                  'non-negative integer', {}),
     }
     priority = -2
 
     @check_messages('unexpected-special-method-signature',
-                    'non-iterator-returned')
+                    'non-iterator-returned', 'invalid-length-returned')
     def visit_functiondef(self, node):
         if not node.is_method():
             return
         if node.name == '__iter__':
             self._check_iter(node)
+        if node.name == '__len__':
+            self._check_len(node)
         if node.name in PYMETHODS:
             self._check_unexpected_method_signature(node)
 
+    visit_asyncfunctiondef = visit_functiondef
+
     def _check_unexpected_method_signature(self, node):
         expected_params = SPECIAL_METHODS_PARAMS[node.name]
 
@@ -997,23 +1055,49 @@ class SpecialMethodsChecker(BaseChecker):
                              args=(node.name, expected_params, current_params, verb),
                              node=node)
 
+    @staticmethod
+    def _is_iterator(node):
+        if node is astroid.YES:
+            # Just ignore YES objects.
+            return True
+        if isinstance(node, Generator):
+            # Generators can be itered.
+            return True
+
+        if isinstance(node, astroid.Instance):
+            try:
+                node.local_attr(NEXT_METHOD)
+                return True
+            except astroid.NotFoundError:
+                pass
+        elif isinstance(node, astroid.ClassDef):
+            metaclass = node.metaclass()
+            if metaclass and isinstance(metaclass, astroid.ClassDef):
+                try:
+                    metaclass.local_attr(NEXT_METHOD)
+                    return True
+                except astroid.NotFoundError:
+                    pass
+        return False
+
     def _check_iter(self, node):
-        try:
-            infered = node.infer_call_result(node)
-        except astroid.InferenceError:
+        infered = _safe_infer_call_result(node, node)
+        if infered is not None:
+            if not self._is_iterator(infered):
+                self.add_message('non-iterator-returned', node=node)
+
+    def _check_len(self, node):
+        inferred = _safe_infer_call_result(node, node)
+        if inferred is None or inferred is astroid.YES:
             return
 
-        for infered_node in infered:
-            if (infered_node is astroid.YES
-                    or isinstance(infered_node, Generator)):
-                continue
-            if isinstance(infered_node, astroid.Instance):
-                try:
-                    infered_node.local_attr(NEXT_METHOD)
-                except astroid.NotFoundError:
-                    self.add_message('non-iterator-returned',
-                                     node=node)
-                    break
+        if not isinstance(inferred, astroid.Const):
+            self.add_message('invalid-length-returned', node=node)
+            return
+
+        value = inferred.value
+        if not isinstance(value, six.integer_types) or value < 0:
+            self.add_message('invalid-length-returned', node=node)
 
 
 def _ancestors_to_call(klass_node, method='__init__'):
