@@ -1,39 +1,44 @@
-# Copyright (c) 2003-2013 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """imports checkers for Python code"""
 
+import collections
 import os
 import sys
-from collections import defaultdict, Counter
+from distutils import sysconfig
 
 import six
 
 import astroid
 from astroid import are_exclusive
-from astroid.modutils import (EXT_LIB_DIR, get_module_part, is_standard_module,
-                              file_from_modpath)
+from astroid.modutils import (get_module_part, is_standard_module)
+import isort
 
 from pylint.interfaces import IAstroidChecker
 from pylint.utils import EmptyReport, get_global_option
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import check_messages, node_ignores_exception
+from pylint.checkers.utils import (
+    check_messages,
+    node_ignores_exception,
+    is_from_fallback_block
+)
 from pylint.graph import get_cycles, DotBackend
 from pylint.reporters.ureports.nodes import VerbatimText, Paragraph
 
 
+def _qualified_names(modname):
+    """Split the names of the given module into subparts
+
+    For example,
+        _qualified_names('pylint.checkers.ImportsChecker')
+    returns
+        ['pylint', 'pylint.checkers', 'pylint.checkers.ImportsChecker']
+    """
+    names = modname.split('.')
+    return ['.'.join(names[0:i+1]) for i in range(len(names))]
+
+
 def _get_import_name(importnode, modname):
     """Get a prepared module name from the given import node
 
@@ -51,7 +56,7 @@ def _get_import_name(importnode, modname):
     return modname
 
 
-def get_first_import(node, context, name, base, level):
+def _get_first_import(node, context, name, base, level, alias):
     """return the node where [base.]<name> is imported or None if not found
     """
     fullname = '%s.%s' % (base, name) if base else name
@@ -68,17 +73,26 @@ def get_first_import(node, context, name, base, level):
                 found = True
                 break
         elif isinstance(first, astroid.ImportFrom):
-            if level == first.level and any(
-                    fullname == '%s.%s' % (first.modname, iname[0])
-                    for iname in first.names):
-                found = True
+            if level != first.level:
+                continue
+
+            for imported_name, imported_alias in first.names:
+                if fullname == '%s.%s' % (first.modname, imported_name):
+                    found = True
+                    break
+                if name != '*' and name == imported_name:
+                    if alias or imported_alias:
+                        continue
+                    found = True
+                    break
+            if found:
                 break
     if found and not are_exclusive(first, node):
         return first
 
 # utilities to represents import dependencies as tree and dot graph ###########
 
-def make_tree_defs(mod_files_list):
+def _make_tree_defs(mod_files_list):
     """get a list of 2-uple (module, list_of_files_which_import_this_module),
     it will return a dictionary to represent this as a tree
     """
@@ -90,7 +104,8 @@ def make_tree_defs(mod_files_list):
         node[1] += files
     return tree_defs
 
-def repr_tree_defs(data, indent_str=None):
+
+def _repr_tree_defs(data, indent_str=None):
     """return a string which represents imports as a tree"""
     lines = []
     nodes = data.items()
@@ -109,11 +124,11 @@ def repr_tree_defs(data, indent_str=None):
             else:
                 sub_indent_str = '%s| ' % indent_str
         if sub:
-            lines.append(repr_tree_defs(sub, sub_indent_str))
+            lines.append(_repr_tree_defs(sub, sub_indent_str))
     return '\n'.join(lines)
 
 
-def dependencies_graph(filename, dep_info):
+def _dependencies_graph(filename, dep_info):
     """write dependencies as a dot (graphviz) file
     """
     done = {}
@@ -132,11 +147,11 @@ def dependencies_graph(filename, dep_info):
     printer.generate(filename)
 
 
-def make_graph(filename, dep_info, sect, gtype):
+def _make_graph(filename, dep_info, sect, gtype):
     """generate a dependencies graph and add some information about it in the
     report's section
     """
-    dependencies_graph(filename, dep_info)
+    _dependencies_graph(filename, dep_info)
     sect.append(Paragraph('%simports graph has been written to %s'
                           % (gtype, filename)))
 
@@ -193,6 +208,11 @@ MSGS = {
               'Used when code and imports are mixed'),
     }
 
+
+DEFAULT_STANDARD_LIBRARY = ()
+DEFAULT_KNOWN_THIRD_PARTY = ('enchant',)
+
+
 class ImportsChecker(BaseChecker):
     """checks for
     * external modules dependencies
@@ -207,7 +227,7 @@ class ImportsChecker(BaseChecker):
     msgs = MSGS
     priority = -2
 
-    if sys.version_info < (3,):
+    if six.PY2:
         deprecated_modules = ('regsub', 'TERMIOS', 'Bastion', 'rexec')
     else:
         deprecated_modules = ('optparse', )
@@ -239,8 +259,30 @@ given file (report RP0402 must not be disabled)'}
                  'help' : 'Create a graph of internal dependencies in the \
 given file (report RP0402 must not be disabled)'}
                ),
+               ('known-standard-library',
+                {'default': DEFAULT_STANDARD_LIBRARY,
+                 'type': 'csv',
+                 'metavar': '<modules>',
+                 'help': 'Force import order to recognize a module as part of' \
+                         ' the standard compatibility libraries.'}
+               ),
+               ('known-third-party',
+                {'default': DEFAULT_KNOWN_THIRD_PARTY,
+                 'type': 'csv',
+                 'metavar': '<modules>',
+                 'help': 'Force import order to recognize a module as part of' \
+                         ' a third party library.'}
+               ),
+               ('analyse-fallback-blocks',
+                {'default': False,
+                 'type': 'yn',
+                 'metavar': '<y_or_n>',
+                 'help': 'Analyse import fallback blocks. This can be used to '
+                         'support both Python 2 and 3 compatible code, which means that '
+                         'the block might have code that exists only in one or another '
+                         'interpreter, leading to false positives when analysed.'}),
+
               )
-    ext_lib_dir = os.path.normcase(os.path.abspath(EXT_LIB_DIR))
 
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
@@ -250,17 +292,40 @@ given file (report RP0402 must not be disabled)'}
         self._first_non_import_node = None
         self.__int_dep_info = self.__ext_dep_info = None
         self.reports = (('RP0401', 'External dependencies',
-                         self.report_external_dependencies),
+                         self._report_external_dependencies),
                         ('RP0402', 'Modules dependencies graph',
-                         self.report_dependencies_graph),
+                         self._report_dependencies_graph),
                        )
 
+        self._site_packages = self._compute_site_packages()
+
+    @staticmethod
+    def _compute_site_packages():
+        def _normalized_path(path):
+            return os.path.normcase(os.path.abspath(path))
+
+        paths = set()
+        real_prefix = getattr(sys, 'real_prefix', None)
+        for prefix in filter(None, (real_prefix, sys.prefix)):
+            path = sysconfig.get_python_lib(prefix=prefix)
+            path = _normalized_path(path)
+            paths.add(path)
+
+        # Handle Debian's derivatives /usr/local.
+        if os.path.isfile("/etc/debian_version"):
+            for prefix in filter(None, (real_prefix, sys.prefix)):
+                libpython = os.path.join(prefix, "local", "lib",
+                                         "python" + sysconfig.get_python_version(),
+                                         "dist-packages")
+                paths.add(libpython)
+        return paths
+
     def open(self):
         """called before visiting project (i.e set of modules)"""
         self.linter.add_stats(dependencies={})
         self.linter.add_stats(cycles=[])
         self.stats = self.linter.stats
-        self.import_graph = defaultdict(set)
+        self.import_graph = collections.defaultdict(set)
         self._ignored_modules = get_global_option(
             self, 'ignored-modules', default=[])
 
@@ -272,48 +337,48 @@ given file (report RP0402 must not be disabled)'}
             for cycle in get_cycles(self.import_graph, vertices=vertices):
                 self.add_message('cyclic-import', args=' -> '.join(cycle))
 
-    @check_messages('wrong-import-position')
+    @check_messages('wrong-import-position', 'multiple-imports',
+                    'relative-import', 'reimported', 'deprecated-module')
     def visit_import(self, node):
         """triggered when an import statement is seen"""
+        self._check_reimport(node)
+
         modnode = node.root()
         names = [name for name, _ in node.names]
         if len(names) >= 2:
             self.add_message('multiple-imports', args=', '.join(names), node=node)
+
         for name in names:
             self._check_deprecated_module(node, name)
-            importedmodnode = self.get_imported_module(node, name)
-            if isinstance(node.scope(), astroid.Module):
+            importedmodnode = self._get_imported_module(node, name)
+            if isinstance(node.parent, astroid.Module):
+                # Allow imports nested
                 self._check_position(node)
+            if isinstance(node.scope(), astroid.Module):
                 self._record_import(node, importedmodnode)
+
             if importedmodnode is None:
                 continue
+
             self._check_relative_import(modnode, node, importedmodnode, name)
             self._add_imported_module(node, importedmodnode.name)
-            self._check_reimport(node, name)
 
-    # TODO This appears to be the list of all messages of the checker...
-    # @check_messages('W0410', 'W0401', 'W0403', 'W0402', 'W0404', 'W0406', 'E0401')
     @check_messages(*(MSGS.keys()))
     def visit_importfrom(self, node):
         """triggered when a from statement is seen"""
         basename = node.modname
-        if basename == '__future__':
-            # check if this is the first non-docstring statement in the module
-            prev = node.previous_sibling()
-            if prev:
-                # consecutive future statements are possible
-                if not (isinstance(prev, astroid.ImportFrom)
-                        and prev.modname == '__future__'):
-                    self.add_message('misplaced-future', node=node)
-            return
+        self._check_misplaced_future(node)
         self._check_deprecated_module(node, basename)
-        for name, _ in node.names:
-            if name == '*':
-                self.add_message('wildcard-import', args=basename, node=node)
+        self._check_wildcard_imports(node)
+        self._check_same_line_imports(node)
+        self._check_reimport(node, basename=basename, level=node.level)
+
         modnode = node.root()
-        importedmodnode = self.get_imported_module(node, basename)
-        if isinstance(node.scope(), astroid.Module):
+        importedmodnode = self._get_imported_module(node, basename)
+        if isinstance(node.parent, astroid.Module):
+            # Allow imports nested
             self._check_position(node)
+        if isinstance(node.scope(), astroid.Module):
             self._record_import(node, importedmodnode)
         if importedmodnode is None:
             return
@@ -322,17 +387,9 @@ given file (report RP0402 must not be disabled)'}
         for name, _ in node.names:
             if name != '*':
                 self._add_imported_module(node, '%s.%s' % (importedmodnode.name, name))
-                self._check_reimport(node, name, basename, node.level)
 
-        # Detect duplicate imports on the same line.
-        names = (name for name, _ in node.names)
-        counter = Counter(names)
-        for name, count in counter.items():
-            if count > 1:
-                self.add_message('reimported', node=node,
-                                 args=(name, node.fromlineno))
-
-    @check_messages('wrong-import-order', 'ungrouped-imports')
+    @check_messages('wrong-import-order', 'ungrouped-imports',
+                    'wrong-import-position')
     def leave_module(self, node):
         # Check imports are grouped by category (standard, 3rd party, local)
         std_imports, ext_imports, loc_imports = self._check_imports_order(node)
@@ -351,7 +408,7 @@ given file (report RP0402 must not be disabled)'}
         self._imports_stack = []
         self._first_non_import_node = None
 
-    def visit_if(self, node):
+    def compute_first_non_import_node(self, node):
         # if the node does not contain an import instruction, and if it is the
         # first node of the module, keep a track of it (all the import positions
         # of the module will be compared to the position of this first
@@ -360,20 +417,70 @@ given file (report RP0402 must not be disabled)'}
             return
         if not isinstance(node.parent, astroid.Module):
             return
-        if any(node.nodes_of_class((astroid.Import, astroid.ImportFrom))):
+        nested_allowed = [astroid.TryExcept, astroid.TryFinally]
+        is_nested_allowed = [
+            allowed for allowed in nested_allowed if isinstance(node, allowed)]
+        if is_nested_allowed and \
+                any(node.nodes_of_class((astroid.Import, astroid.ImportFrom))):
             return
+        if isinstance(node, astroid.Assign):
+            # Add compatibility for module level dunder names
+            # https://www.python.org/dev/peps/pep-0008/#module-level-dunder-names
+            valid_targets = [
+                isinstance(target, astroid.AssignName) and
+                target.name.startswith('__') and target.name.endswith('__')
+                for target in node.targets]
+            if all(valid_targets):
+                return
         self._first_non_import_node = node
 
-    visit_tryfinally = visit_tryexcept = visit_assignattr = visit_assign \
-            = visit_ifexp = visit_comprehension = visit_if
+    visit_tryfinally = visit_tryexcept = visit_assignattr = visit_assign = \
+        visit_ifexp = visit_comprehension = visit_expr = visit_if = \
+        compute_first_non_import_node
 
     def visit_functiondef(self, node):
-        # if it is the first non import instruction of the module, record it
-        if not self._first_non_import_node:
-            self._first_non_import_node = node
+        # If it is the first non import instruction of the module, record it.
+        if self._first_non_import_node:
+            return
+
+        # Check if the node belongs to an `If` or a `Try` block. If they
+        # contain imports, skip recording this node.
+        if not isinstance(node.parent.scope(), astroid.Module):
+            return
+
+        root = node
+        while not isinstance(root.parent, astroid.Module):
+            root = root.parent
+
+        if isinstance(root, (astroid.If, astroid.TryFinally, astroid.TryExcept)):
+            if any(root.nodes_of_class((astroid.Import, astroid.ImportFrom))):
+                return
+
+        self._first_non_import_node = node
 
     visit_classdef = visit_for = visit_while = visit_functiondef
 
+    def _check_misplaced_future(self, node):
+        basename = node.modname
+        if basename == '__future__':
+            # check if this is the first non-docstring statement in the module
+            prev = node.previous_sibling()
+            if prev:
+                # consecutive future statements are possible
+                if not (isinstance(prev, astroid.ImportFrom)
+                        and prev.modname == '__future__'):
+                    self.add_message('misplaced-future', node=node)
+            return
+
+    def _check_same_line_imports(self, node):
+        # Detect duplicate imports on the same line.
+        names = (name for name, _ in node.names)
+        counter = collections.Counter(names)
+        for name, count in counter.items():
+            if count > 1:
+                self.add_message('reimported', node=node,
+                                 args=(name, node.fromlineno))
+
     def _check_position(self, node):
         """Check `node` import or importfrom node position is correct
 
@@ -389,9 +496,27 @@ given file (report RP0402 must not be disabled)'}
         """Record the package `node` imports from"""
         importedname = importedmodnode.name if importedmodnode else None
         if not importedname:
-            importedname = node.names[0][0].split('.')[0]
+            if isinstance(node, astroid.ImportFrom):
+                importedname = node.modname
+            else:
+                importedname = node.names[0][0].split('.')[0]
+        if isinstance(node, astroid.ImportFrom) and (node.level or 0) >= 1:
+            # We need the impotedname with first point to detect local package
+            # Example of node:
+            #  'from .my_package1 import MyClass1'
+            #  the output should be '.my_package1' instead of 'my_package1'
+            # Example of node:
+            #  'from . import my_package2'
+            #  the output should be '.my_package2' instead of '{pyfile}'
+            importedname = '.' + importedname
         self._imports_stack.append((node, importedname))
 
+    @staticmethod
+    def _is_fallback_import(node, imports):
+        imports = [import_node for (import_node, _) in imports]
+        return any(astroid.are_exclusive(import_node, node)
+                   for import_node in imports)
+
     def _check_imports_order(self, node):
         """Checks imports of module `node` are grouped by category
 
@@ -400,43 +525,44 @@ given file (report RP0402 must not be disabled)'}
         extern_imports = []
         local_imports = []
         std_imports = []
-        stdlib_paths = [sys.prefix, self.ext_lib_dir]
-        real_prefix = getattr(sys, 'real_prefix', None)
-        if real_prefix is not None:
-            stdlib_paths.append(real_prefix)
-
+        extern_not_nested = []
+        local_not_nested = []
+        isort_obj = isort.SortImports(
+            file_contents='', known_third_party=self.config.known_third_party,
+            known_standard_library=self.config.known_standard_library,
+        )
         for node, modname in self._imports_stack:
-            package = modname.split('.')[0]
-            if is_standard_module(modname):
-                std_imports.append((node, package))
-                wrong_import = extern_imports or local_imports
-                if not wrong_import:
-                    continue
-                self.add_message('wrong-import-order', node=node,
-                                 args=('standard import "%s"' % node.as_string(),
-                                       '"%s"' % wrong_import[0][0].as_string()))
+            if modname.startswith('.'):
+                package = '.' + modname.split('.')[1]
             else:
-                try:
-                    filename = file_from_modpath([package])
-                except ImportError:
-                    extern_imports.append((node, package))
-                    continue
-                if not filename:
-                    extern_imports.append((node, package))
-                    continue
-                filename = os.path.normcase(os.path.abspath(filename))
-                if not any(filename.startswith(path) for path in stdlib_paths):
-                    local_imports.append((node, package))
+                package = modname.split('.')[0]
+            nested = not isinstance(node.parent, astroid.Module)
+            import_category = isort_obj.place_module(package)
+            if import_category in ('FUTURE', 'STDLIB'):
+                std_imports.append((node, package))
+                wrong_import = extern_not_nested or local_not_nested
+                if self._is_fallback_import(node, wrong_import):
                     continue
+                if wrong_import and not nested:
+                    self.add_message('wrong-import-order', node=node,
+                                     args=('standard import "%s"' % node.as_string(),
+                                           '"%s"' % wrong_import[0][0].as_string()))
+            elif import_category in ('FIRSTPARTY', 'THIRDPARTY'):
                 extern_imports.append((node, package))
-                if not local_imports:
-                    continue
-                self.add_message('wrong-import-order', node=node,
-                                 args=('external import "%s"' % node.as_string(),
-                                       '"%s"' % local_imports[0][0].as_string()))
+                if not nested:
+                    extern_not_nested.append((node, package))
+                wrong_import = local_not_nested
+                if wrong_import and not nested:
+                    self.add_message('wrong-import-order', node=node,
+                                     args=('external import "%s"' % node.as_string(),
+                                           '"%s"' % wrong_import[0][0].as_string()))
+            elif import_category == 'LOCALFOLDER':
+                local_imports.append((node, package))
+                if not nested:
+                    local_not_nested.append((node, package))
         return std_imports, extern_imports, local_imports
 
-    def get_imported_module(self, importnode, modname):
+    def _get_imported_module(self, importnode, modname):
         try:
             return importnode.do_import_module(modname)
         except astroid.InferenceError as ex:
@@ -446,25 +572,16 @@ given file (report RP0402 must not be disabled)'}
             else:
                 args = repr(dotted_modname)
 
-            for submodule in self._qualified_names(modname):
+            for submodule in _qualified_names(modname):
                 if submodule in self._ignored_modules:
                     return None
 
+            if not self.config.analyse_fallback_blocks and is_from_fallback_block(importnode):
+                return None
+
             if not node_ignores_exception(importnode, ImportError):
                 self.add_message("import-error", args=args, node=importnode)
 
-    @staticmethod
-    def _qualified_names(modname):
-        """Split the names of the given module into subparts
-
-        For example,
-            _qualified_names('pylint.checkers.ImportsChecker')
-        returns
-            ['pylint', 'pylint.checkers', 'pylint.checkers.ImportsChecker']
-        """
-        names = modname.split('.')
-        return ['.'.join(names[0:i+1]) for i in range(len(names))]
-
     def _check_relative_import(self, modnode, importnode, importedmodnode,
                                importedasname):
         """check relative import. node is either an Import or From node, modname
@@ -486,14 +603,26 @@ given file (report RP0402 must not be disabled)'}
 
     def _add_imported_module(self, node, importedmodname):
         """notify an imported module, used to analyze dependencies"""
+        module_file = node.root().file
+        context_name = node.root().name
+        base = os.path.splitext(os.path.basename(module_file))[0]
+
+        # Determine if we have a `from .something import` in a package's
+        # __init__. This means the module will never be able to import
+        # itself using this condition (the level will be bigger or
+        # if the same module is named as the package, it will be different
+        # anyway).
+        if isinstance(node, astroid.ImportFrom):
+            if node.level and node.level > 0 and base == '__init__':
+                return
+
         try:
             importedmodname = get_module_part(importedmodname,
-                                              node.root().file)
+                                              module_file)
         except ImportError:
             pass
-        context_name = node.root().name
+
         if context_name == importedmodname:
-            # module importing itself !
             self.add_message('import-self', node=node)
         elif not is_standard_module(importedmodname):
             # handle dependencies
@@ -512,7 +641,7 @@ given file (report RP0402 must not be disabled)'}
             if mod_path == mod_name or mod_path.startswith(mod_name + '.'):
                 self.add_message('deprecated-module', node=node, args=mod_path)
 
-    def _check_reimport(self, node, name, basename=None, level=None):
+    def _check_reimport(self, node, basename=None, level=None):
         """check if the import is necessary (i.e. not already done)"""
         if not self.linter.is_message_enabled('reimported'):
             return
@@ -522,22 +651,23 @@ given file (report RP0402 must not be disabled)'}
         contexts = [(frame, level)]
         if root is not frame:
             contexts.append((root, None))
+
         for context, level in contexts:
-            first = get_first_import(node, context, name, basename, level)
-            if first is not None:
-                self.add_message('reimported', node=node,
-                                 args=(name, first.fromlineno))
+            for name, alias in node.names:
+                first = _get_first_import(node, context, name, basename, level, alias)
+                if first is not None:
+                    self.add_message('reimported', node=node,
+                                     args=(name, first.fromlineno))
 
-
-    def report_external_dependencies(self, sect, _, dummy):
+    def _report_external_dependencies(self, sect, _, dummy):
         """return a verbatim layout for displaying dependencies"""
-        dep_info = make_tree_defs(six.iteritems(self._external_dependencies_info()))
+        dep_info = _make_tree_defs(six.iteritems(self._external_dependencies_info()))
         if not dep_info:
             raise EmptyReport()
-        tree_str = repr_tree_defs(dep_info)
+        tree_str = _repr_tree_defs(dep_info)
         sect.append(VerbatimText(tree_str))
 
-    def report_dependencies_graph(self, sect, _, dummy):
+    def _report_dependencies_graph(self, sect, _, dummy):
         """write dependencies as a dot (graphviz) file"""
         dep_info = self.stats['dependencies']
         if not dep_info or not (self.config.import_graph
@@ -546,15 +676,15 @@ given file (report RP0402 must not be disabled)'}
             raise EmptyReport()
         filename = self.config.import_graph
         if filename:
-            make_graph(filename, dep_info, sect, '')
+            _make_graph(filename, dep_info, sect, '')
         filename = self.config.ext_import_graph
         if filename:
-            make_graph(filename, self._external_dependencies_info(),
-                       sect, 'external ')
+            _make_graph(filename, self._external_dependencies_info(),
+                        sect, 'external ')
         filename = self.config.int_import_graph
         if filename:
-            make_graph(filename, self._internal_dependencies_info(),
-                       sect, 'internal ')
+            _make_graph(filename, self._internal_dependencies_info(),
+                        sect, 'internal ')
 
     def _external_dependencies_info(self):
         """return cached external dependencies information or build and
@@ -580,6 +710,11 @@ given file (report RP0402 must not be disabled)'}
                     result[importee] = importers
         return self.__int_dep_info
 
+    def _check_wildcard_imports(self, node):
+        for name, _ in node.names:
+            if name == '*':
+                self.add_message('wildcard-import', args=node.modname, node=node)
+
 
 def register(linter):
     """required method to auto register this checker """
