@@ -1,18 +1,6 @@
-# Copyright (c) 2006-2013 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """try to find more bugs in the code using astroid inference capabilities
 """
 
@@ -41,12 +29,7 @@ from pylint.checkers.utils import (
     supports_subscript,
     safe_infer,
     has_known_bases)
-from pylint import utils
 
-
-_ZOPE_DEPRECATED = (
-    "This option is deprecated. Use generated-members instead."
-)
 BUILTINS = six.moves.builtins.__name__
 STR_FORMAT = "%s.str.format" % BUILTINS
 
@@ -281,19 +264,19 @@ class should be ignored. A mixin class is detected if its name ends with \
                          'deduced by static analysis. It supports qualified '
                          'module names, as well as Unix pattern matching.'}
                ),
+               # the defaults here are *stdlib* names that (almost) always
+               # lead to false positives, since their idiomatic use is
+               # 'too dynamic' for pylint to grok.
                ('ignored-classes',
-                {'default' : (),
+                {'default' : ('optparse.Values', 'thread._local', '_thread._local'),
                  'type' : 'csv',
                  'metavar' : '<members names>',
-                 'help' : 'List of classes names for which member attributes '
+                 'help' : 'List of class names for which member attributes '
                           'should not be checked (useful for classes with '
-                          'attributes dynamically set). This supports '
-                          'can work with qualified names.'}
+                          'dynamically set attributes). This supports '
+                          'the use of qualified names.'}
                ),
 
-               ('zope', utils.deprecated_option(opt_type='yn',
-                                                help_msg=_ZOPE_DEPRECATED)),
-
                ('generated-members',
                 {'default' : (),
                  'type' : 'string',
@@ -302,11 +285,28 @@ class should be ignored. A mixin class is detected if its name ends with \
 missed by pylint inference system, and so shouldn\'t trigger E1101 when \
 accessed. Python regular expressions are accepted.'}
                ),
+               ('contextmanager-decorators',
+                {'default': ['contextlib.contextmanager'],
+                 'type': 'csv',
+                 'metavar': '<decorator names>',
+                 'help': 'List of decorators that produce context managers, '
+                         'such as contextlib.contextmanager. Add to this list '
+                         'to register other decorators that produce valid '
+                         'context managers.'}
+               ),
               )
 
     def open(self):
         # do this in open since config not fully initialized in __init__
-        self.generated_members = list(self.config.generated_members)
+        # generated_members may contain regular expressions
+        # (surrounded by quote `"` and followed by a comma `,`)
+        # REQUEST,aq_parent,"[a-zA-Z]+_set{1,2}"' =>
+        # ('REQUEST', 'aq_parent', '[a-zA-Z]+_set{1,2}')
+        if isinstance(self.config.generated_members, six.string_types):
+            gen = shlex.shlex(self.config.generated_members)
+            gen.whitespace += ','
+            gen.wordchars += '[]-+\.*?'
+            self.config.generated_members = tuple(tok.strip('"') for tok in gen)
 
     def visit_assignattr(self, node):
         if isinstance(node.assign_type(), astroid.AugAssign):
@@ -324,19 +324,13 @@ accessed. Python regular expressions are accepted.'}
 
         function/method, super call and metaclasses are ignored
         """
-        # generated_members may contain regular expressions
-        # (surrounded by quote `"` and followed by a comma `,`)
-        # REQUEST,aq_parent,"[a-zA-Z]+_set{1,2}"' =>
-        # ('REQUEST', 'aq_parent', '[a-zA-Z]+_set{1,2}')
-        if isinstance(self.config.generated_members, str):
-            gen = shlex.shlex(self.config.generated_members)
-            gen.whitespace += ','
-            gen.wordchars += '[]-+'
-            self.config.generated_members = tuple(tok.strip('"') for tok in gen)
         for pattern in self.config.generated_members:
             # attribute is marked as generated, stop here
             if re.match(pattern, node.attrname):
                 return
+            if re.match(pattern, node.as_string()):
+                return
+
         try:
             infered = list(node.expr.infer())
         except exceptions.InferenceError:
@@ -468,6 +462,33 @@ accessed. Python regular expressions are accepted.'}
                                      args=node.func.as_string())
                     break
 
+    @staticmethod
+    def _no_context_variadic(node):
+        """Verify if the given call node has variadic nodes without context
+
+        This is a workaround for handling cases of nested call functions
+        which don't have the specific call context at hand.
+        Variadic arguments (variable positional arguments and variable
+        keyword arguments) are inferred, inherently wrong, by astroid
+        as a Tuple, respectively a Dict with empty elements.
+        This can lead pylint to believe that a function call receives
+        too few arguments.
+        """
+        for arg in node.args:
+            if not isinstance(arg, astroid.Starred):
+                continue
+
+            inferred = safe_infer(arg.value)
+            if isinstance(inferred, astroid.Tuple):
+                length = len(inferred.elts)
+            elif isinstance(inferred, astroid.Dict):
+                length = len(inferred.items)
+            else:
+                return False
+            if not length and isinstance(inferred.statement(), astroid.FunctionDef):
+                return True
+        return False
+
     @check_messages(*(list(MSGS.keys())))
     def visit_call(self, node):
         """check that called functions/methods are inferred to callable objects,
@@ -479,6 +500,7 @@ accessed. Python regular expressions are accepted.'}
         call_site = astroid.arguments.CallSite.from_call(node)
         num_positional_args = len(call_site.positional_arguments)
         keyword_args = list(call_site.keyword_arguments.keys())
+        no_context_variadic = self._no_context_variadic(node)
 
         called = safe_infer(node.func)
         # only function, generator and object defining __call__ are allowed
@@ -494,6 +516,7 @@ accessed. Python regular expressions are accepted.'}
             # Any error occurred during determining the function type, most of
             # those errors are handled by different warnings.
             return
+
         num_positional_args += implicit_args
         if called.args.args is None:
             # Built-in functions have no argument information.
@@ -610,8 +633,10 @@ accessed. Python regular expressions are accepted.'}
                     display_name = '<tuple>'
                 else:
                     display_name = repr(name)
-                self.add_message('no-value-for-parameter', node=node,
-                                 args=(display_name, callable_name))
+                # TODO(cpopa): this should be removed after PyCQA/astroid/issues/177
+                if not no_context_variadic:
+                    self.add_message('no-value-for-parameter', node=node,
+                                     args=(display_name, callable_name))
 
         for name in kwparams:
             defval, assigned = kwparams[name]
@@ -635,6 +660,8 @@ accessed. Python regular expressions are accepted.'}
         parent_type = safe_infer(node.parent.value)
         if not isinstance(parent_type, (astroid.ClassDef, astroid.Instance)):
             return
+        if not has_known_bases(parent_type):
+            return
 
         # Determine what method on the parent this index will use
         # The parent of this node will be a Subscript, and the parent of that
@@ -740,7 +767,8 @@ accessed. Python regular expressions are accepted.'}
             if isinstance(infered, bases.Generator):
                 # Check if we are dealing with a function decorated
                 # with contextlib.contextmanager.
-                if decorated_with(infered.parent, ['contextlib.contextmanager']):
+                if decorated_with(infered.parent,
+                                  self.config.contextmanager_decorators):
                     continue
                 # If the parent of the generator is not the context manager itself,
                 # that means that it could have been returned from another
@@ -756,7 +784,8 @@ accessed. Python regular expressions are accepted.'}
                     scope = path.scope()
                     if not isinstance(scope, astroid.FunctionDef):
                         continue
-                    if decorated_with(scope, ['contextlib.contextmanager']):
+                    if decorated_with(scope,
+                                      self.config.contextmanager_decorators):
                         break
                 else:
                     self.add_message('not-context-manager',
