@@ -1,16 +1,6 @@
-# Copyright (c) 2003-2013 LOGILAB S.A. (Paris, FRANCE).
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """utilities methods and classes for reporters"""
 from __future__ import print_function
 
@@ -55,19 +45,7 @@ class BaseReporter(object):
         self.path_strip_prefix = os.getcwd() + os.sep
 
     def handle_message(self, msg):
-        """Handle a new message triggered on the current file.
-
-        Invokes the legacy add_message API by default."""
-        self.add_message(
-            msg.msg_id, (msg.abspath, msg.module, msg.obj, msg.line, msg.column),
-            msg.msg)
-
-    def add_message(self, msg_id, location, msg):
-        """Deprecated, do not use."""
-        # pylint: disable=no-self-use,unused-argument
-        msg = ("This method is deprecated, use handle_message instead. "
-               "It will be removed in Pylint 1.6.")
-        warnings.warn(msg, DeprecationWarning, stacklevel=2)
+        """Handle a new message triggered on the current file."""
 
     def set_output(self, output=None):
         """set output stream"""
